#include "CSRFile.h"

CSRFile::CSRFile()
{
	ZeroMemory(&m_InfoHeader,sizeof(m_InfoHeader));
}
CSRFile::~CSRFile()
{
	m_file.CloseFile();
}
BOOL CSRFile::WriteFileCrc32AtEnd(LPSTR lpFileName)
{
	DWORD crc;
	CFileIO file;

	if ( ! FileCrc32(lpFileName, &crc, NULL,NULL) )
		return FALSE;

	if ( !file.OpenFile(lpFileName) )
			return FALSE;

	if ( ! file.SeekToEOF() )
			return FALSE;
			
	if ( ! file.WriteFile(&crc,sizeof(DWORD)) )
		return FALSE;

	file.CloseFile();

	return TRUE;
}
BOOL CSRFile::CreateNewFile(LPSTR lpFileName,srfInfoHeader * pFileHeader)
{
	LPSTR lpBuffer;

	if ( pFileHeader->dwRecordSize == NULL || pFileHeader->dwHeaderSize > MAX_DATA_HEADER_SIZE)
		return FALSE;

	if ( ! m_file.CreateNewFile(lpFileName) )
		return FALSE;

	if ( ! m_file.WriteFile(SRF_HEADER,sizeof(SRF_HEADER)) )
		return FALSE;

	m_InfoHeader = *pFileHeader;
	
	if ( ! m_file.WriteFile(&m_InfoHeader,sizeof(m_InfoHeader)) )
		return FALSE;

	//... Write Crc32 after Header and Info Header
	m_file.CloseFile();
	if ( !WriteFileCrc32AtEnd(lpFileName) )
		return FALSE;
		
	if ( ! m_file.OpenFile(lpFileName) )
			return FALSE;
	if ( ! m_file.SeekToEOF() )
			return FALSE;
	//...

	if ( m_InfoHeader.dwHeaderSize )
	{
		lpBuffer = new char [m_InfoHeader.dwHeaderSize];
		ZeroMemory(lpBuffer, m_InfoHeader.dwHeaderSize);

		if ( ! m_file.WriteFile(lpBuffer,m_InfoHeader.dwHeaderSize) )
		{
			delete [] lpBuffer;
			return FALSE;
		}
		delete [] lpBuffer;	
	}
	//... Write Crc32 after Header ,Info Header And Data Header
	m_file.CloseFile();
	if ( !WriteFileCrc32AtEnd(lpFileName) )
		return FALSE;
	//...

	//Write Crc32 of file at its end
	if ( !WriteFileCrc32AtEnd(lpFileName) )
		return FALSE;
	if ( !OpenFile(lpFileName) )
		return FALSE;
	return TRUE;
}
BOOL CSRFile::OpenFile(LPSTR lpFileName,srfInfoHeader * pFileHeader )
{
  BOOL bRet;

  ZeroMemory(&m_InfoHeader,sizeof(m_InfoHeader));	

  bRet = OpenFile(lpFileName);

  if ( !bRet )
  {
	  m_file.CloseFile();
	  ZeroMemory(&m_InfoHeader,sizeof(m_InfoHeader));	
	  return FALSE;
  }
	
  if (m_InfoHeader.dwFileID == pFileHeader->dwFileID && m_InfoHeader.dwHeaderSize == pFileHeader->dwHeaderSize && m_InfoHeader.dwRecordSize == pFileHeader->dwRecordSize )
	  return TRUE;

	m_file.CloseFile();
	ZeroMemory(&m_InfoHeader,sizeof(m_InfoHeader));	
	return FALSE;
}
//...
BOOL CSRFile::OpenFile(LPSTR lpFileName)
{
	if (!IsHeaderOk(lpFileName) )
			return FALSE;

	if ( !m_file.OpenFile(lpFileName) )
			return FALSE;

	// Read Info Header
	srfInfoHeader ihData;
	
	if ( !m_file.SetFilePointer(GetDataLocation(LOC_INFO_HEADER)) )
		return FALSE;
	if ( !m_file.ReadFile(&ihData, sizeof(ihData)) )
		 return FALSE;

	m_InfoHeader = ihData;

	return TRUE;
}
BOOL CSRFile::IsHeaderOk(LPSTR lpFileName)
{
	CFileIO file;
	
	if ( ! file.OpenReadOnly(lpFileName) )
			return FALSE;
	char chHeader[sizeof(SRF_HEADER) + 1];
	chHeader[sizeof(SRF_HEADER)] = 0;
	
	if ( ! file.ReadFile(&chHeader,sizeof(SRF_HEADER)) )
		return FALSE;
	
	if ( lstrcmp(chHeader, SRF_HEADER ) != 0 )
		return FALSE;

file.CloseFile();
return TRUE;
}
BOOL CSRFile::GetFileCrc32Sized(CFileIO * pfile,DWORD dwSizeLimit,
					   LPDWORD lpCrc32,LPBOOL lpCancel,HWND hwndPB)
{
	//CFileIO file;

	LPSTR lpBuffer;
	DWORD dwCrc = InitializeCrc32();
	DWORD dwFileSize;
	DWORD dwRead ,dwWholeReads = 0;
	BOOL  bStop = FALSE;
	LPBOOL lpStop;
	DWORD dwRemaining;
	
	if (lpCancel)
		lpStop = lpCancel;
	else
		lpStop = &bStop;

	if (! pfile->SeekToBOF() )
		return FALSE;

	if ( ! dwSizeLimit )
		return FALSE;

	dwFileSize = dwSizeLimit;
	dwRemaining = dwSizeLimit;
	lpBuffer = new char [FILE_READ_BUFFER_SIZE];
	do
	{
		if(dwRemaining >= FILE_READ_BUFFER_SIZE)
		{
		dwRead = pfile->ReadFile(lpBuffer,FILE_READ_BUFFER_SIZE);
		dwRemaining -= dwRead;
		}
		else
		{
		dwRead = pfile->ReadFile(lpBuffer,dwRemaining);
		dwRemaining = 0;
		}

	
		
		if (dwRead)
			dwCrc = PrepareCrc32(lpBuffer,dwRead,dwCrc);
		
		dwWholeReads += dwRead;
		if (hwndPB)
			SendMessage(hwndPB,PBM_SETPOS,GetPercent(dwWholeReads,dwFileSize),0);
	} while (dwRead > 0 && *lpStop == FALSE && dwRemaining);
	
	//file.CloseFile();
	delete [] lpBuffer;

	if ( dwWholeReads != dwFileSize )
			return FALSE;

	*lpCrc32 = FinalCrc32(dwCrc);
	return TRUE;
}
BOOL CSRFile::IsFileOk(LPSTR lpFileName, LPBOOL lpCancel,HWND hwndPB)
{
	// Check the header before CRC32 verification
	if (!IsHeaderOk(lpFileName) )
			return FALSE;

	DWORD dwFileCrc,dwNewCrc;
	CFileIO file;
	DWORD dwFileSize;
	
	if ( !file.OpenReadOnly(lpFileName) )
			return FALSE;

	if ( !file.SetFilePointer(file.GetFileSize() - sizeof(DWORD)) )
			return FALSE;

	if ( !file.ReadFile(&dwFileCrc, sizeof(DWORD)) )
			return FALSE;

	dwFileSize = file.GetFileSize() - sizeof(DWORD);

	//....
	if ( !GetFileCrc32Sized(&file,dwFileSize,&dwNewCrc,lpCancel,hwndPB) )
		return FALSE;

	file.CloseFile();

	if ( dwNewCrc == dwFileCrc)
		return TRUE;

	return FALSE;
}

BOOL CSRFile::CloseFile(void)
{
	ZeroMemory(&m_InfoHeader,sizeof(m_InfoHeader));
	return m_file.CloseFile();
}

BOOL CSRFile::WriteFileHeader(LPCVOID lpBuffer)
{
	if ( !m_InfoHeader.dwHeaderSize )
		return FALSE;

	if ( IsBadReadPtr(lpBuffer, m_InfoHeader.dwHeaderSize) )
		return FALSE;

	if ( !m_file.SetFilePointer(GetDataLocation(LOC_DATA_HEADER)) )
		return FALSE;

	if ( !m_file.WriteFile(lpBuffer,m_InfoHeader.dwHeaderSize) )
		return FALSE;
	
	//... CRC32
	DWORD dwNewCrc,dwFileSize;
	dwFileSize = GetDataLocation(LOC_DATA_HEADER) + m_InfoHeader.dwHeaderSize;
	if ( !GetFileCrc32Sized(&m_file,dwFileSize,&dwNewCrc,NULL,NULL) )
		return FALSE;
	
	if ( !m_file.SetFilePointer(dwFileSize) )
		return FALSE;
	
	if ( !m_file.WriteFile(&dwNewCrc,sizeof(dwNewCrc)) )
		return FALSE;

return TRUE;
}

BOOL CSRFile::ReadFileHeader(LPVOID lpBuffer)
{
	if ( !m_InfoHeader.dwHeaderSize )
		return FALSE;

	if ( IsBadWritePtr(lpBuffer, m_InfoHeader.dwHeaderSize) )
		return FALSE;


	if ( !m_file.SetFilePointer(GetDataLocation(LOC_DATA_HEADER)) )
		return FALSE;
	
	if ( !m_file.ReadFile(lpBuffer,m_InfoHeader.dwHeaderSize) )
		return FALSE;
	//...
	// crc check
		DWORD dwCrc,dwNewCrc,dwFileSize;
	
	if ( !m_file.ReadFile(&dwCrc,sizeof(DWORD)) )
		return FALSE;

		dwFileSize = GetDataLocation(LOC_DATA_HEADER) + m_InfoHeader.dwHeaderSize;
	if ( !GetFileCrc32Sized(&m_file,dwFileSize,&dwNewCrc,NULL,NULL) )
		return FALSE;
	
	if ( dwCrc != dwNewCrc )
		return FALSE;
	//...
return TRUE;
}
DWORD CSRFile::GetDataLocation(INT iDataType)
{
	switch (iDataType)
	{
	case LOC_INFO_HEADER:
		return sizeof(SRF_HEADER);
	case LOC_DATA_HEADER:
		return sizeof(SRF_HEADER) + sizeof(m_InfoHeader) + sizeof(DWORD);
	case LOC_FIRST_RECORD:
		return sizeof(SRF_HEADER) + sizeof(m_InfoHeader) + sizeof(DWORD) + m_InfoHeader.dwHeaderSize + sizeof(DWORD);
	case LOC_LAST_RECORD:
		if (GetNumberOfRecord() == NULL )
			return GetDataLocation(LOC_FIRST_RECORD);
		return GetDataLocation(LOC_FIRST_RECORD) + ( (GetRecordSize()  * GetNumberOfRecord() ) );
	}
	return NULL;
}
DWORD CSRFile::GetRecordSize(void)
{
	// RID + CRC32 + Data
return  sizeof(DWORD) + sizeof(DWORD) + m_InfoHeader.dwRecordSize ;
}

BOOL CSRFile::WriteNewRecord(LPCVOID lpBuffer)
{
	if ( IsBadReadPtr(lpBuffer, m_InfoHeader.dwRecordSize) )
		return FALSE;
	

	if ( !m_file.SetFilePointer(GetDataLocation(LOC_LAST_RECORD)) )
		return FALSE;

	//... RID + CRC32 write phase
	DWORD dwRID,dwCRC32;
	dwRID = RID_VALUE;
	dwCRC32 = BufferCrc32( (LPSTR) lpBuffer,m_InfoHeader.dwRecordSize); 
	
	if ( !m_file.WriteFile(&dwRID, sizeof(DWORD) ) )
		return FALSE;
	
	if ( !m_file.WriteFile(&dwCRC32, sizeof(DWORD) ) )
		return FALSE;

	//... Data write phase
	if ( !m_file.WriteFile(lpBuffer,m_InfoHeader.dwRecordSize) )
		return FALSE;

	// write a dummy crc before calling function WriteFileCrc32
	dwCRC32 = NULL;	
	if ( !m_file.WriteFile(&dwCRC32, sizeof(DWORD) ) )
		return FALSE;

return WriteFileCrc32(&m_file);
}

BOOL CSRFile::ReWriteRecord(LPCVOID lpBuffer,DWORD dwRecNumber)
{
	DWORD dwLoc = GetDataLocation(LOC_FIRST_RECORD) + (GetRecordSize() *  ( dwRecNumber - 1));
	
	if ( dwRecNumber < 1 || dwRecNumber > GetNumberOfRecord() )
		return FALSE;

	if ( IsBadReadPtr(lpBuffer, m_InfoHeader.dwRecordSize) )
		return FALSE;

	if ( !m_file.SetFilePointer(dwLoc) )
		return FALSE;

	//... RID + CRC32 write phase
	DWORD dwRID,dwCRC32;
	dwRID = RID_VALUE;
	dwCRC32 = BufferCrc32( (LPSTR) lpBuffer,m_InfoHeader.dwRecordSize); 
	
	if ( !m_file.WriteFile(&dwRID, sizeof(DWORD) ) )
		return FALSE;
	
	if ( !m_file.WriteFile(&dwCRC32, sizeof(DWORD) ) )
		return FALSE;

	//... Data write phase
	if ( !m_file.WriteFile(lpBuffer,m_InfoHeader.dwRecordSize) )
		return FALSE;

	//... CRC32
	return WriteFileCrc32(&m_file);
}
BOOL CSRFile::WriteFileCrc32(CFileIO * pfile)
{
	DWORD dwNewCrc,dwFileSize;
	dwFileSize = pfile->GetFileSize() - sizeof(DWORD);
	if ( !GetFileCrc32Sized(pfile,dwFileSize,&dwNewCrc,NULL,NULL) )
		return FALSE;
	
	if ( !pfile->SetFilePointer(dwFileSize) )
		return FALSE;
	
	if ( !pfile->WriteFile(&dwNewCrc,sizeof(DWORD)) )
		return FALSE;

return TRUE;
}

BOOL CSRFile::DeleteRecord(DWORD dwRecNumber)
{
	DWORD dwLoc = GetDataLocation(LOC_FIRST_RECORD) ;
	dwLoc	+= (GetRecordSize() *  ( dwRecNumber - 1));

	if ( dwRecNumber < 1 || dwRecNumber > GetNumberOfRecord() )
		return FALSE;

	//if ( !m_file.SetFilePointer(dwLoc) )

	if ( !MoveFileDataBack(&m_file, dwLoc, GetRecordSize()) )
		return FALSE;

	return WriteFileCrc32(&m_file);
}
BOOL CSRFile::MoveFileDataBack(CFileIO * pfile, DWORD dwStartLoc, DWORD dwTrashDataSize)
{
	//pfile, dwStartLoc, dwTrashDataSize
	
	LPSTR lpBuffer;
	DWORD nReqRead;
	DWORD dwRead;
	DWORD dwRemaining;
	DWORD nFileSize = pfile->GetFileSize();
	DWORD nWholeReads = 0;

	nReqRead = pfile->GetFileSize() - ( dwStartLoc + dwTrashDataSize );
	dwRemaining = nReqRead;
	lpBuffer = new char [FILE_READ_BUFFER_SIZE];

	do
	{
		//... Seek for Read
		if ( !pfile->SetFilePointer(nFileSize - dwRemaining) )
		{
				delete [] lpBuffer;
				return FALSE;
		}
		//... 
		if(dwRemaining >= FILE_READ_BUFFER_SIZE)
		{
		dwRead = pfile->ReadFile(lpBuffer,FILE_READ_BUFFER_SIZE);
		dwRemaining -= dwRead;
		}
		else
		{
		dwRead = pfile->ReadFile(lpBuffer,dwRemaining);
		dwRemaining = 0;
		}

	
		
		if (dwRead)
		{
		//... Seek for Write
		if ( !pfile->SetFilePointer( ( (nFileSize - dwRemaining) - dwRead) - dwTrashDataSize) )
		{
				delete [] lpBuffer;
				return FALSE;
		}
		//... 
			if ( !pfile->WriteFile(lpBuffer, dwRead) )
			{
				delete [] lpBuffer;
				return FALSE;
			}
		}
		nWholeReads += dwRead;
	} while (dwRead > 0 && dwRemaining);
	
	delete [] lpBuffer;

	if ( nWholeReads != nReqRead )
			return FALSE;

	if ( ! pfile->SetFilePointer(nFileSize - dwTrashDataSize) )
			return FALSE;

	return pfile->SetEndOfFile();
}
BOOL CSRFile::ReadRecord(LPVOID lpBuffer,DWORD dwRecNumber)
{
	DWORD dwLoc = GetDataLocation(LOC_FIRST_RECORD) ;
	dwLoc	+= (GetRecordSize() *  ( dwRecNumber - 1));

	if ( dwRecNumber < 1 || dwRecNumber > GetNumberOfRecord() )
		return FALSE;

	if ( IsBadWritePtr(lpBuffer, m_InfoHeader.dwRecordSize) )
		return FALSE;

	if ( !m_file.SetFilePointer(dwLoc) )
		return FALSE;
	//,,,
	DWORD dwRID ,dwCrc ,dwNewCrc;
	if ( !m_file.ReadFile(&dwRID,sizeof(DWORD) ) )
		return FALSE;

	if ( !m_file.ReadFile(&dwCrc,sizeof(DWORD) ) )
		return FALSE;
	//...

	if ( !m_file.ReadFile(lpBuffer,m_InfoHeader.dwRecordSize) )
		return FALSE;

	//...
	dwNewCrc = BufferCrc32((LPSTR)lpBuffer,m_InfoHeader.dwRecordSize);
	
	if ( dwRID != RID_VALUE || dwNewCrc != dwCrc )
			return FALSE;

return TRUE;
}

DWORD CSRFile::GetNumberOfRecord(void)
{
	if ( GetDataLocation(LOC_FIRST_RECORD) + sizeof(DWORD) >= m_file.GetFileSize() )
		return NULL;
	DWORD dwRecords = m_file.GetFileSize() - (GetDataLocation(LOC_FIRST_RECORD) + sizeof(DWORD));

	return dwRecords / GetRecordSize();
}

srfInfoHeader CSRFile::GetFileInfo(void)
{
	return m_InfoHeader;
}
/*
This file is a part of CIF.
Project Core Information Foundation (CIF).
Developed by Muhammad Arshad Latti.
Location Sargodha (Pakistan).
arshadlatti@gmail.com
http://arshadlatti.angelfire.com/
*/
