//Sized Record File
#ifndef __C_SR_FILE_H__
#define __C_SR_FILE_H__

#include <windows.h>
#include <commctrl.h>
#include "CFileIO.h"
#include "cifcrc.h"
#include "cif_common.h"

#define MAX_DATA_HEADER_SIZE 33554432 // 32MB
#define FILE_READ_BUFFER_SIZE 4096 // 4KB
#define SRF_HEADER "SRF 1.0"
#define RID_VALUE  0x00FF00FF

// File Identification Header = "srf 1.0" + One Null Byte
// After Header Custom Data Information Header = srfInfoHeader
struct srfInfoHeader
{
	DWORD dwFileID;
	DWORD dwHeaderSize;
	DWORD dwRecordSize;
};

// For Function GetDataLocation
#define	LOC_INFO_HEADER 1 
#define	LOC_DATA_HEADER 2
#define	LOC_FIRST_RECORD 3 // A byte before the first record 
#define LOC_LAST_RECORD 4  // A byte before the end of last record
//....

class CSRFile
{
 CFileIO m_file;
 srfInfoHeader m_InfoHeader;
 
public:
	CSRFile();
	~CSRFile();
	BOOL CreateNewFile(LPSTR lpFileName,srfInfoHeader * pFileHeader);
	BOOL IsFileOk(LPSTR lpFileName, LPBOOL lpCancel,HWND hwndPB);
	BOOL IsHeaderOk(LPSTR lpFileName);
	BOOL OpenFile(LPSTR lpFileName,srfInfoHeader * pFileHeader );
	
	BOOL CloseFile(void);

	BOOL WriteFileHeader(LPCVOID lpBuffer);
	
	BOOL ReadFileHeader(LPVOID lpBuffer);

	BOOL WriteNewRecord(LPCVOID lpBuffer);

	BOOL ReWriteRecord(LPCVOID lpBuffer,DWORD dwRecNumber);

	BOOL DeleteRecord(DWORD dwRecNumber);
	BOOL ReadRecord(LPVOID lpBuffer,DWORD dwRecNumber);

	DWORD GetNumberOfRecord(void);

	srfInfoHeader GetFileInfo(void);
	
private:
		BOOL WriteFileCrc32AtEnd(LPSTR lpFileName);
		BOOL WriteFileCrc32(CFileIO * pfile);
		DWORD GetDataLocation(INT iDataType);
		DWORD GetRecordSize(void);
		BOOL GetFileCrc32Sized(CFileIO * pfile, DWORD dwSizeLimit,LPDWORD lpCrc32,LPBOOL lpCancel,HWND hwndPB);
		BOOL OpenFile(LPSTR lpFileName);
		BOOL MoveFileDataBack(CFileIO * pfile, DWORD dwStartLoc, DWORD dwTrashDataSize);
};
#endif
/*
This file is a part of CIF.
Project Core Information Foundation (CIF).
Developed by Muhammad Arshad Latti.
Location Sargodha (Pakistan).
arshadlatti@gmail.com
http://arshadlatti.angelfire.com/
*/
